/*
*	This was written by Arun Sagar and is placed into the public domain.
*	The author hereby disclaims copyright to this source code.
*
*	Pay it forward. 
*/

#ifdef DUMPANALYSIS
	#include <fstream.h>
#endif

class cRFastBoard : public cReversiBoard
{
typedef cReversiBoard super;

private:
	typedef int8 StabilityData;
	typedef CaptureFlag XCaptureFlag[8];

private:
	static StabilityData grgStability[];
	static XCaptureFlag grgXCaptureY[8][256];
	static ulong gMinmaxMoveCounterTotal;

#ifdef DUMPANALYSIS
	static ofstream ofsDump;
	static bool fDumpMoveAnalysis;
	static const char *kszDefaultDumpFile;
#endif

private:
	LongLine BoardY[8];
	LongLine BoardYX[15];
	LongLine BoardXY[15];


	CaptureFlag LegalMovesTopFiller[8];
	CaptureFlag LegalMoves[8];
	CaptureFlag LegalMovesBottomFiller[8];

private:
	inline void StuffRow(LongLine rLine, Position column);
	inline void StuffColumn(LongLine rLine, Position column);
	void StuffXY(LongLine rLine, Position xy);
	void StuffYX(LongLine rLine, Position yx);
	inline void SetPiece(Piece, Position, Position);
	inline void StuffColumnFlag(CaptureFlag rFlag, Position column);
	inline void StuffXYFlag(CaptureFlag rFlag, Position xy);
	inline void StuffYXFlag(CaptureFlag rFlag, Position yx);

	inline long Stability(Piece) const ;
	inline long CountMobility(void) const ;
	void UpdateMobility(Piece);
	bool NextValidMove(Piece, Position*, Position*) const ;

#ifdef DEBUG
	void CrosscheckBoard(void);
	ulong FullLineCount(void);
	ulong EmptyLineCount(void);
#endif

public:
	long (cRFastBoard :: *pStrengthFunc)(Piece) const;

protected:
	void Clear(void);

public:
	static bool Initialize(void);
	
	cRFastBoard(){pStrengthFunc=0;}
	cRFastBoard(const cReversiBoard&);

	void MakeMove(Piece, Position, Position);

	long Strength(Piece) const ;
	long LosingStrength(Piece) const ;
	long PositionalStrength(Piece) const ;
	long GreedyStrength(Piece) const ;

	long ABPrune(Piece, long, long, long, const bool *);
	//long EndGameSearch(Piece, ulong, long, long, const bool *);

#ifdef DUMPANALYSIS
	friend ostream& operator<<(ostream&, const cRFastBoard&);
#endif
	
friend class cRFTable;
friend class cRPlayer;
};

